/**
 * PETALS - PETALS Services Platform. Copyright (c) 2007 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.bc.sql.listener;

import java.io.IOException;
import java.io.PipedInputStream;
import java.io.PipedOutputStream;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.ow2.petals.bc.sql.service.SQLService;

/**
 * The SQL Response is written in a PipeStream an thread while the response of
 * is sent by the JBIListener thread.
 * 
 * @author alouis
 * 
 */
public class OutStreamer implements Runnable {

    private final PipedOutputStream pos = new PipedOutputStream();

    private final PipedInputStream pis = new PipedInputStream();

    private final ResultSet resultSet;

    private final SQLService sqlService;

    private final boolean showMetadata;

    private final Logger logger;

    private final Connection connection;

    /**
     * A Thread used to push resultSet data into a Pipe Stream after the
     * SqlComponent returns the response
     * 
     * @param sqlService
     * @param resultSet
     *            resultSet to transform into an XML doc
     * @param showMetadata
     *            row index, column name and type written as xml attributes
     * @param connection
     *            this connection is closed when all the resultSet has been
     *            written
     * @param logger
     * @throws IOException
     */
    public OutStreamer(SQLService sqlService, ResultSet resultSet, boolean showMetadata,
            Connection connection, Logger logger) throws IOException {
        super();
        this.sqlService = sqlService;
        this.showMetadata = showMetadata;
        this.logger = logger;
        this.resultSet = resultSet;
        this.connection = connection;
        this.pos.connect(this.pis);
    }

    public PipedInputStream getPipedInputStream() {
        return this.pis;
    }

    public void run() {
        try {
            this.logger.finest("Build the response of the 'select' operation");
            this.sqlService.resultSetAsXMLOutputStream(this.resultSet, this.pos,
                    this.showMetadata);
            this.pos.close();

        } catch (final Exception e) {
            this.logger.log(Level.SEVERE, "(OutStreamer Thread) - " + e.getMessage(), e);
        } finally {
            // close the connection
            try {
                this.connection.close();
            } catch (final SQLException e) {
                this.logger.log(Level.SEVERE, e.getMessage(), e);
            }
        }

    }
}
