/**
 * PETALS - PETALS Services Platform. Copyright (c) 2007 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */
package org.ow2.petals.bc.sql.service;

import java.io.OutputStream;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.logging.Logger;

import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamWriter;

import org.ow2.petals.bc.sql.Constants;
import org.ow2.petals.commons.threadlocal.XMLOutputFactories;

/**
 * 
 * @author alouis
 * 
 */
public class SQLService {

    private Logger logger;

    public SQLService(Logger logger) {
        super();
        this.logger = logger;
    }

    /**
     * Execute the select SQL command on the given connection.
     * 
     * @param sql
     * @param connection
     * @return
     * @throws SQLException
     */
    public ResultSet select(String sql, Connection connection) throws SQLException {
        Statement statement = connection.createStatement();
        logger.info("Execute the following SQL instruction : " + sql);
        
        return statement.executeQuery(sql);
    }

    /**
     * Execute the update SQL command on the given connection.
     * 
     * @param sql
     * @param connection
     * @return
     * @throws SQLException
     */
    public int update(String sql, Connection connection) throws SQLException {
        Statement statement = connection.createStatement();
        logger.info("Execute the following SQL instruction : " + sql);

        return statement.executeUpdate(sql);
    }

    /**
     * Transform the given resultSet as an XML outputStream
     * 
     * @param resultSet
     *            the jdbc resultSet to transform
     * @param outputStream
     *            the stream where the XML is written
     * @param showAttributes
     *            row index, column name and type are set as xml attributes in
     *            the response
     * @throws SQLException
     * @throws XMLStreamException
     */
    public void resultSetAsXMLOutputStream(ResultSet resultSet, OutputStream outputStream,
            boolean showAttributes) throws SQLException, XMLStreamException {
        XMLStreamWriter writer = XMLOutputFactories.getDefaultXMLOutputFactory()
                .createXMLStreamWriter(outputStream);
        writer.writeStartDocument();
        writer.writeStartElement(Constants.RESULT_ELEMENT);
        writer.writeDefaultNamespace(Constants.NAMESPACE);

        if (resultSet != null) {
            ResultSetMetaData metaData = resultSet.getMetaData();
            int columnCount = metaData.getColumnCount();
            // resultSet.beforeFirst();
            while (resultSet.next()) {
                writer.writeStartElement(Constants.ROW);
                if (showAttributes) {
                    writer.writeAttribute(Constants.INDEX, "" + resultSet.getRow());
                }
                for (int i = 1; i <= columnCount; i++) {
                    writer.writeStartElement(Constants.COLUMN);
                    if (showAttributes) {
                        writer.writeAttribute(Constants.NAME, metaData.getColumnName(i));
                        writer.writeAttribute(Constants.TYPE, metaData.getColumnTypeName(i));
                    }
                    writer.writeCharacters(resultSet.getString(i));
                    writer.writeEndElement();
                }
                writer.writeEndElement();
            }
        }
        writer.writeEndElement();
        writer.writeEndDocument();
    }

}
