/**
 * PETALS - PETALS Services Platform. Copyright (c) 2005 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * ------------------------------------------------------------------------- $Id
 * : $ -------------------------------------------------------------------------
 */

package org.ow2.petals.bc.sql.su;

import java.util.List;

import javax.sql.DataSource;

import org.ow2.petals.bc.sql.DataSources;
import org.ow2.petals.bc.sql.SqlComponent;
import org.ow2.petals.component.framework.AbstractComponent;
import org.ow2.petals.component.framework.api.configuration.ConfigurationExtensions;
import org.ow2.petals.component.framework.api.exception.PEtALSCDKException;
import org.ow2.petals.component.framework.jbidescriptor.generated.Jbi;
import org.ow2.petals.component.framework.jbidescriptor.generated.Provides;
import org.ow2.petals.component.framework.su.AbstractServiceUnitManager;
import org.ow2.petals.component.framework.su.ServiceUnitDataHandler;

/**
 * Service Unit Manager for the SQL component.
 * 
 * @author alouis
 */
public class SUManager extends AbstractServiceUnitManager {

    private DataSources dataSources;

    public SUManager(AbstractComponent component) {
        super(component);
    }

    @Override
    public void doDeploy(String serviceUnitName, String suRootPath, Jbi descriptor)
            throws PEtALSCDKException {

        final ServiceUnitDataHandler suDatahandler = this.getServiceUnitDataHandlers().get(
                serviceUnitName);

        if (descriptor.getServices().getConsumes().size() != 0) {
            throw new PEtALSCDKException("Consumes section are not supported by this component");
        }

        for (final Provides provides : descriptor.getServices().getProvides()) {
            this.logger.info("Create a dataSource for " + provides.getServiceName() + "-"
                    + provides.getEndpointName());

            final ConfigurationExtensions extensions = suDatahandler
                    .getConfigurationExtensions(provides);

            final DataSource dataSource = this.getDataSources().createDataSourceForService(
                    extensions);
            this.getDataSources().addDataSource(provides, dataSource);
        }

    }

    @Override
    public void doUndeploy(String serviceUnitName) throws PEtALSCDKException {
        final Jbi descriptor = this.getServiceUnitDataHandlers().get(serviceUnitName)
                .getDescriptor();
        final List<Provides> providesList = descriptor.getServices().getProvides();

        for (final Provides provides : providesList) {
            final DataSource dataSource = this.getDataSources().removeDataSource(provides);
            this.logger.info("Close the dataSource for " + provides.getServiceName() + "-"
                    + provides.getEndpointName());
            this.getDataSources().closeDataSource(dataSource);
        }
    }

    /**
     * No need of synchronized, as only 1 thread work with the CDK SU Manager.
     * 
     * @return
     */
    private DataSources getDataSources() {
        if (this.dataSources == null) {
            this.dataSources = ((SqlComponent) this.component).getDataSources();
        }

        return this.dataSources;
    }
}
